@@ -1,11 +1,13 @@
 """views module."""
+from argparse import Namespace
 from collections import Counter
 from types import SimpleNamespace
+from typing import Any, List, Optional, Tuple
 from urllib.parse import urlparse
 import itertools
 import logging
 
-from flask import flash, redirect, request, url_for
+from flask import current_app, flash, redirect, request, url_for
 from flask_admin.babel import gettext
 from flask_admin.base import AdminIndexView, BaseView, expose
 from flask_admin.model import BaseModelView
@@ -75,7 +77,8 @@ class BookmarkModelView(BaseModelView):
     def _create_ajax_loader(self, name, options):
         pass
 
-    def _list_entry(self, context, model, name):
+    def _list_entry(
+            self, context: Any, model: Namespace, name: str) -> Markup:
         parsed_url = urlparse(model.url)
         netloc, scheme = parsed_url.netloc, parsed_url.scheme
         is_scheme_valid = scheme in ('http', 'https')
@@ -91,11 +94,16 @@ class BookmarkModelView(BaseModelView):
             """.format(
                 model, ''.join(tag_text), Markup.escape(model.url)
             ))
-        netloc_tmpl = '<img src="{}{}"/> '
-        res = netloc_tmpl.format(
-            'http://www.google.com/s2/favicons?domain=', netloc)
+        res = ''
+        if not current_app.config.get('BUKUSERVER_DISABLE_FAVICON', False):
+            netloc_tmpl = '<img src="{}{}"/> '
+            res = netloc_tmpl.format(
+                'http://www.google.com/s2/favicons?domain=', netloc)
         title = model.title if model.title else '&lt;EMPTY TITLE&gt;'
-        if is_scheme_valid:
+        open_in_new_tab = current_app.config.get('BUKUSERVER_OPEN_IN_NEW_TAB', False)
+        if is_scheme_valid and open_in_new_tab:
+            res += '<a href="{0.url}" target="_blank">{1}</a>'.format(model, title)
+        elif is_scheme_valid and not open_in_new_tab:
             res += '<a href="{0.url}">{1}</a>'.format(model, title)
         else:
             res += title
@@ -381,23 +389,29 @@ class TagModelView(BaseModelView):
 
     def scaffold_form(self):
         class CustomForm(FlaskForm):  # pylint: disable=too-few-public-methods
-            name = wtforms.StringField(validators=[wtforms.validators.required()])
+            name = wtforms.StringField(validators=[wtforms.validators.DataRequired()])
 
         return CustomForm
 
     def scaffold_list_form(self, widget=None, validators=None):
         pass
 
-    def get_list(self, page, sort_field, sort_desc, search, filters, page_size=None):
+    def get_list(
+            self,
+            page: int,
+            sort_field: str,
+            sort_desc: bool,
+            search: Optional[Any],
+            filters: List[Tuple[int, str, str]],
+            page_size: int = None) -> Tuple[int, List[SimpleNamespace]]:
         bukudb = self.bukudb
         tags = bukudb.get_tag_all()[1]
-        tags = [(x, y) for x, y in tags.items()]
+        tags = sorted(tags.items())
         tags = self._apply_filters(tags, filters)
-        if sort_field == 'usage_count':
-            tags = sorted(tags, key=lambda x: x[1], reverse=sort_desc)
-        elif sort_field == 'name':
-            tags = sorted(tags, key=lambda x: x[0], reverse=sort_desc)
-        tags = list(tags)
+        sort_field_dict = {'usage_count': 1, 'name': 0}
+        if sort_field in sort_field_dict:
+            tags = list(sorted(
+                tags, key=lambda x: x[sort_field_dict[sort_field]], reverse=sort_desc))
         count = len(tags)
         if page_size and tags:
             tags = list(chunks(tags, page_size))[page]
